<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://atareao.es
 * @since      1.0.0
 *
 * @package    Rester
 * @subpackage Rester/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Rester
 * @subpackage Rester/admin
 * @author     Lorenzo Carbonell <lorenzo.carbonell.cerezo@gmail.com>
 */
class Rester_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Rester_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Rester_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/rester-admin.css', array(), $this->version, 'all' );

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Rester_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Rester_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/rester-admin.js', array( 'jquery' ), $this->version, false );

    }

    public function register_rest_routes()
    {
        register_rest_route('rester/v1', 'auth/validate', array(
            'methods'             => 'GET',
            'callback'            => array($this, 'rester_auth_validate'),
            'permission_callback' => array($this, 'check_access_token')
        ));
        register_rest_route('rester/v1', '/speakers', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_speakers'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/speaker/(?P<id>\d+)', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_speakers'),
            'permission_callback' => array($this, 'access_allowed'),
            'args'     => array(
                'id'   => array(
                    'validate_callback' => function($param, $request, $key){
                        return is_numeric($param);
                    }
                )),
        ));
        register_rest_route('rester/v1', '/sponsors', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_sponsors'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/sponsor/(?P<id>\d+)', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_sponsors'),
            'permission_callback' => array($this, 'access_allowed'),
            'args'     => array(
                'id'   => array(
                    'validate_callback' => function($param, $request, $key){
                        return is_numeric($param);
                    }
                )),
        ));
        register_rest_route('rester/v1', '/basic_sessions', array(
            'methods' => 'GET',
            'callback' => array($this, 'basic_sessions'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/sessions', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_sessions'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/session/(?P<id>\d+)', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_sessions'),
            'permission_callback' => array($this, 'access_allowed'),
            'args'     => array(
                'id'   => array(
                    'validate_callback' => function($param, $request, $key){
                        return is_numeric($param);
                    }
                )),
        ));
        register_rest_route('rester/v1', '/rooms', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_rooms'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/room/(?P<id>\d+)', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_rooms'),
            'permission_callback' => array($this, 'access_allowed'),
            'args'     => array(
                'id'   => array(
                    'validate_callback' => function($param, $request, $key){
                        return is_numeric($param);
                    }
                )),
        ));
        register_rest_route('rester/v1', '/organizers', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_organizers'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/products', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_products'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/orders', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_orders'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/assistants', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_assistants'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/tickets', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_tickets'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/organizer/(?P<id>\d+)', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_organizers'),
            'permission_callback' => array($this, 'access_allowed'),
            'args'     => array(
                'id'   => array(
                    'validate_callback' => function($param, $request, $key){
                        return is_numeric($param);
                    }
                )),
        ));
        register_rest_route('rester/v1', '/testimonials', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_testimonials'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
        register_rest_route('rester/v1', '/testimonial/(?P<id>\d+)', array(
            'methods'  => 'GET',
            'callback' => array($this, 'rester_testimonials'),
            'permission_callback' => array($this, 'access_allowed'),
            'args'     => array(
                'id'   => array(
                    'validate_callback' => function($param, $request, $key){
                        return is_numeric($param);
                    }
                )),
        ));
        register_rest_route('rester/v1', '/schedules', array(
            'methods' => 'GET',
            'callback' => array($this, 'rester_schedules'),
            'permission_callback' => array($this, 'access_allowed'),
        ));
    }

    public function access_allowed(WP_REST_Request $request)
    {
        return TRUE;
    }

    private function get_user_from_token($header)
    {
        try {
            if(!str_contains($header, " ")){
                return FALSE;
            }
            list($kind, $token) = explode(" ", $header);
            if(strtolower($kind) == "bearer"){
                if(class_exists("Jwt_Auth_Public")){
                    $auth = new Jwt_Auth_Public("jwt-auth", "1.1.0");
                    $response = $auth->validate_token(FALSE);
                    return $response->data->user->id;
                }
            }
        } catch (\Throwable $th) {
            error_log($th->getMessage());
        }
        return NULL;

    }

    public function check_access_token(WP_REST_Request $request)
    {
        $header = trim($request->get_header('Authorization'));
        $token = $this->get_user_from_token($header);
        return isset($token);
    }

    public function check_access(WP_REST_Request $request)
    {
        $header = trim($request->get_header('Authorization'));
        try {
            if(!str_contains($header, " ")){
                return FALSE;
            }
            list($kind, $token) = explode(" ", $header);
            if(strtolower($kind) == "basic"){
                list($user_mail, $password) = explode(":", base64_decode($token));
                $user = get_user_by('email', $user_mail);
                if(!empty($user)){
                    return TRUE;
                }
            }
        } catch (\Throwable $th) {
            error_log($th->getMessage());
        }
        return FALSE;
    }

    public function rester_auth($request)
    {
        $header = $request->get_header("Authorization");
        return "Cabecera $header";
        $id = $this->get_user_from_token($header);
        $user = get_user_by("id", $id);
        $user_email = $user->user_email;
        return "Hola mundo $user_email";
    }

    public function rester_auth_validate($request)
    {
        $header = $request->get_header("Authorization");
        $id = $this->get_user_from_token($header);
        if(isset($id)){
            $response = [
                "code" => "jwt_auth_valid_token",
                "data" => [ "status" => 200]
            ];
            return new WP_HTTP_Response($response, 200);
        }
        $response = [
            "code" => "jwt_auth_invalid_token",
            "message" => "Signature verification failed",
            "data"    => [ "status" => 403]
        ];
        return new WP_HTTP_Response($response, 403);
    }

    public function rester_speakers($data)
    {
        $id = NULL;
        if(isset($data)){
            $id = $data['id'];
        }
        return $this->get('speaker', [
            'id'        => 'ID',
            'custom_id' => 'post_name',
            'name'      => 'post_title',
            'content'   => 'post_content',
            'facebook'  => 'facebook_link',
            'instagram' => 'instagram_link',
            'twitter'   => 'twitter_link',
            'linkedin'  => 'linkedIn_link',
            'company'   => 'company',
            'website'   => 'website_link'
            ],
            $id,
        );
    }

    public function rester_sponsors($data)
    {
        $id = NULL;
        if(isset($data)){
            $id = $data['id'];
        }
        return $this->get_sponsors($id);
    }

    public function basic_sessions()
    {
        $args = array(
            'posts_per_page' => -1,
            'post_type'      => 'session'
        );
        $query = new WP_Query($args);
        return $query->get_posts();
    }

    public function rester_sessions($data)
    {
        $id = NULL;
        if(isset($data)){
            $id = $data['id'];
        }
        return $this->get_sessions($id);
    }

    public function rester_rooms($data)
    {
        $id = NULL;
        if(isset($data)){
            $id = $data['id'];
        }
        return $this->get('room', [
            'id'        => 'ID',
            'name'      => 'post_title',
            'content'   => 'post_content',
            'custom_id' => 'post_name'
            ],
            $id,
        );
    }

    public function rester_testimonials($data)
    {
        $id = NULL;
        if(isset($data)){
            $id = $data['id'];
        }
        return $this->get('testimonial', [
            'id' => 'ID',
            'name' => 'post_title',
            'content'   => 'post_content',
            'facebook'  => 'facebook_link',
            'instagram' => 'instagram_link',
            'twitter'   => 'twitter_link',
            'linkedin'  => 'linkedIn_link',
            'company'   => 'company',
            'website'   => 'website_link'
            ],
            $id,
        );
    }

    public function rester_organizers($data)
    {
        $id = NULL;
        if(isset($data)){
            $id = $data['id'];
        }
        return $this->get('wep_organizer', [
            'id' => 'ID',
            'name' => 'post_title',
            'content'   => 'post_content',
            'facebook'  => 'facebook_link',
            'instagram' => 'instagram_link',
            'twitter'   => 'twitter_link',
            'linkedin'  => 'linkedIn_link'
            ],
            $id,
        );
    }
    public function rester_tickets()
    {
        return $this->get_all_debug('tribe_events', []);
    }
    public function rester_products()
    {
        $response = [];
        $args = array(
            'limit' => -1
        );
        try{
            $products = wc_get_products($args);
            foreach ($products as $product) {
                $response[] = [
                    "id" => $product->id,
                    "name" => $product->get_name(),
                ];
            }
        }catch(Exception $e){
            $response[] = ["error" => $e->getMessage()];

        }
        return $response;
    }

    public function rester_orders()
    {
        $response = [];
        $args = array(
            'limit' => -1
        );
        try{
            $orders = wc_get_orders($args);
            foreach ($orders as $order) {
                $response[] = [
                    "id" => $order->id,
                    "customer_id" => $order->get_customer_id(),
                    "key" => $order->get_order_key(),
                    "data" => $order->get_meta_data(),
                    "base_data" => $order->get_base_data(),
                    "meta_data" => $order->get_meta_data(),
                    "user" => $order->get_user()
                ];
            }
        }catch(Exception $e){
            $response[] = ["error" => $e->getMessage()];

        }
        return $response;
    }

    private function get_data($data, $email){
        $response = [];
        $response["name"] = "";
        $response["talla"] = "";
        $response["tipo"] = "";
        $response["notificacion"] = "";

        $items = $data["870"];
        $items = json_decode(json_encode($items, TRUE));
        $response["tipo"] = gettype($items);
        $response["count"] = $items[0];
        return $response;
        foreach ($items as $info){
            $response["first"] = $info;
            return $response;
        //for($i = 0; $i < count($data["870"]); $i++){
            //$info = $data["870"][$i];
            if($info["tribe-tickets-plus-iac-email"] == $email){
                $response["name"] = $this->get_default($info, "tribe-tickets-plus-iac-name");
                $response["talla"] = $this->get_default($info, "talla-de-camiseta");
                $response["tipo"] = $this->get_default($info, "tipo-visitante");
                $response["notificacion"] = $this->get_default($info, "si-eres-podcaster-quieres-que-notifiquemos-a-tus-seguidores-y-mostremos-tu-llegada-en-los-monitores");
                break;
            }
        }
        return $response;
    }

    private function get_default($item, $field){
        if(array_key_exists($field, $item)){
            return $item[$field];
        }
        return "";
    }

    public function rester_assistants()
    {
        $response = [];
        $args = array(
            'limit' => -1
        );
        try{
            $orders = wc_get_orders($args);
            foreach ($orders as $order) {
                $assistant = [];
                $user_id = $order->get_customer_id();
                $user = get_user_by("id", $user_id);
                $email = $user->get("user_email");
                $assistant["id"] = $user_id;
                $assistant["email"] = $email;
                $items = $order->get_meta_data();
                foreach($items as $item){
                    if($item->key == "_tribe_tickets_meta"){

                        $info = $item->value;
                        $content = $this->get_data($item->value, $email);
                        $assistant["content"] = $content;
                        $assistant["info"] = $info;
                        /*
                        $assistant["name"] = $content["name"];
                        $assistant["tipo"] = $content["tipo"];
                        $assistant["talla"] = $content["talla"];
                        $assistant["notificacion"] = $content["notificacion"];
                        $assistant["info"] = $info;
                        */
                        break;
                    }
                }
                $response[] = $assistant;
            }
        }catch(Exception $e){
            $response[] = ["error" => $e->getMessage()];

        }
        return $response;
    }


    private function get_all_debug($post_type, $params)
    {
        $args = array(
            'posts_per_page' => -1,
            'post_type'      => $post_type,
        );
        $query = new WP_Query($args);
        $items = $query->get_posts();
        foreach($items as $key => $item)
        {
            $custom = get_post_custom($item->ID);
            $item = json_decode(json_encode($item), TRUE);
            $custom = json_decode(json_encode($custom), TRUE);
            $custom = array_map(function($item){
                return $item[0];
            }, $custom);
            $items[$key] = array_merge($item, $custom);
        }
        return $items;
    }

    public function get_items_by_custom_id($post_type)
    {
        $response = [];
        $items = $this->get_all($post_type, [
                'id'        => 'ID',
                'custom_id' => 'post_name'
            ]);
        foreach ($items as $item) {
            $id = $item["id"];
            $custom_id = $item["custom_id"];
            $response[$custom_id] = $id;
        }
        return $response;
    }

    public function rester_schedules()
    {
        $result = array();
        $scheduleSession[] = '';
        $scheduleDate = get_option('session_date') ;
        $scheduleStart = get_option('session_time_start') ;
        $scheduleEnds= get_option('session_time_end') ;
        $scheduleSession = get_option('selectSession') ;
        $roomSession = get_option('selectRoom');
        if(!empty($scheduleSession))
        {
            $sessions = $this->get_sessions_by_custom_id();
            $rooms = $this->get_rooms_by_custom_id();
            for($i=0; $i<count($scheduleSession); $i++)
            {
                if(array_key_exists($scheduleSession[$i], $sessions))
                {
                    $session = $sessions[$scheduleSession[$i]];
                }
                else
                {
                    $session = NULL;
                }
                if(array_key_exists($roomSession[$i], $rooms))
                {
                    $room = $rooms[$roomSession[$i]];
                }
                else
                {
                    $room = NULL;
                }
                $result[] = [
                    'schedule_date'     => $scheduleDate[$i],
                    'schedule_start'    => $scheduleStart[$i],
                    'schedule_end'      => $scheduleEnds[$i],
                    'session'           => $session,
                    'room'              => $room,
                ];
            }
        }
        return $result;
    }



    private function get_speakers_by_custom_id()
    {
        $items = $this->rester_speakers(NULL);
        $response = [];
        foreach ($items as $item) {
            $custom_id = $item["custom_id"];
            $response[$custom_id] = $item;
        }
        return $response;
    }

    private function get_rooms_by_custom_id()
    {
        $items = $this->rester_rooms(NULL);
        $response = [];
        foreach ($items as $item) {
            $custom_id = $item["custom_id"];
            $response[$custom_id] = $item;
        }
        return $response;
    }

    private function get_sessions_by_custom_id()
    {
        $items = $this->get_sessions();
        $response = [];
        foreach ($items as $item) {
            $custom_id = $item["custom_id"];
            $response[$custom_id] = $item;
        }
        return $response;

    }

    private function get_sessions($id=NULL)
    {
        $result = array();
        $args = array(
            'posts_per_page' => -1,
            'post_type'      => 'session'
        );
        if(isset($id))
        {
            $args['id'] = explode("-", $id)[0];
        }
        $query = new WP_Query($args);
        $items = $query->get_posts();
        $speakers_by_cid = $this->get_speakers_by_custom_id();
        foreach($items as $item)
        {
            $custom = get_post_custom($item->ID);
            $item = json_decode(json_encode($item), TRUE);
            $custom = json_decode(json_encode($custom), TRUE);
            $custom = array_map(function($item){
                return $item[0];
            }, $custom);
            $item = array_merge($item, $custom);

            $one = [];
            $one["id"]        = $item["ID"];
            $one["custom_id"] = $item["post_name"];
            $one["name"]      = $item["post_title"];
            $one["content"]   = $item["post_content"];
            $one["speakers"]  = [];
            if(key_exists("speakers", $item) && isset($item["speakers"]))
            {
                $speakers = unserialize($item["speakers"]);
                if($speakers !== FALSE){
                    foreach ($speakers as $speaker) {
                        $speaker_custom_id = sanitize_title($speaker);
                        if(array_key_exists($speaker_custom_id, $speakers_by_cid))
                        {
                            $one["speakers"][] = $speakers_by_cid[$speaker_custom_id];
                        }else{
                            $one["speakers"][] = $speaker_custom_id;
                        }
                    }
                }
            }
            $result[] = $one;
        }
        return $result;
    }

    private function get_sponsors($id=NULL)
    {
        $result = array();
        $args = array(
            'posts_per_page' => -1,
            'post_type'      => 'sponsor'
        );
        if(isset($id))
        {
            $args['id'] = explode("-", $id)[0];
        }
        $query = new WP_Query($args);
        $items = $query->get_posts();
        foreach($items as $item)
        {
            $level_id = $item->ID;
            $custom = get_post_custom($item->ID);
            $item = json_decode(json_encode($item), TRUE);
            $custom = json_decode(json_encode($custom), TRUE);
            $custom = array_map(function($item){
                return $item[0];
            }, $custom);
            $item = array_merge($item, $custom);
            $names = unserialize($item['image_name']);
            $descriptions = unserialize($item['photo_desc']);
            $logos = unserialize($item['txt_image_url']);
            $urls = unserialize($item['image_link']);
            $total = count($names);
            for ($i=0; $i < $total; $i++) {
                $one = [];
                $one["index"] = "$level_id-$i";
                $one["category"] = $item["post_title"];
                $one["name"] = isset($names[$i])?$names[$i]:"";
                $one["description"] = isset($descriptions[$i])?$descriptions[$i]:"";
                $one["logo"] = isset($logos[$i])?$logos[$i]:"";
                $one["url"] = isset($urls[$i])?$urls[$i]:"";
                if(isset($id) && $one["index"] == $id){
                    return $one;
                }
                if(isset($one["name"]) && !empty($one["name"]))
                {
                    $result[] = $one;
                }
            }
        }
        return $result;
    }

    private function get($post_type, $params, $id=NULL)
    {
        if(isset($id))
        {
            return $this->get_one($post_type, $params, $id);
        }
        else
        {
            return $this->get_all($post_type, $params);
        }
    }

    private function get_all($post_type, $params)
    {
        $result = array();
        $args = array(
            'posts_per_page' => -1,
            'post_type'      => $post_type,
        );
        $query = new WP_Query($args);
        $items = $query->get_posts();
        foreach($items as $item)
        {
            $id = $item->ID;
            $custom = get_post_custom($item->ID);
            $item = json_decode(json_encode($item), TRUE);
            $custom = json_decode(json_encode($custom), TRUE);
            $custom = array_map(function($item){
                return $item[0];
            }, $custom);
            $item = array_merge($item, $custom);
            foreach($params as $key => $value)
            {
                $one[$key] = $item[$value];
            }
            $one["photo"] = get_the_post_thumbnail_url($id);
            $result[] = $one;
        }
        return $result;
    }

    private function get_one($post_type, $params, $id)
    {
        $args = array(
            'post_type' => $post_type,
            'ID'        => $id,
        );
        $query = new WP_Query($args);
        if($query->have_posts())
        {
            $items = $query->get_posts();
            $custom = get_post_custom($items[0]->ID);
            $custom = json_decode(json_encode($custom), TRUE);
            $custom = array_map(function($item){
                return $item[0];
            }, $custom);
            $item = json_decode(json_encode($items[0]), TRUE);
            $item = array_merge($item, $custom);
            foreach($params as $key => $value)
            {
                $one[$key] = $item[$value];
            }
            $one["photo"] = get_the_post_thumbnail_url($id);
            return $one;
        }
        return new WP_Error('not_found', 'Not found', array('status' => 404));
    }
}
